﻿//
// © Algobox Ltd - Benn Eichhorn
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using Algobox.Objects.Finance.Assets.Stock;
using Algobox.Feeds.Finance.MarketData;

namespace Algobox.Structure.Finance.Strategies.Terrapin
{
    public class TerrapinUnitSummary : INotifyPropertyChanged, IMarketUpdateLast
    {
        #region Constructors

        public TerrapinUnitSummary(Strategy strategy)
        {
            if (Strategy.Security.Currency == Objects.Finance.Currencies.Currency.GBX)
                _divisor = 100;
            else
                _divisor = 1;
        }

        #endregion
        //////////////////
        #region Member Variables

        public event PropertyChangedEventHandler PropertyChanged;

        public TerrapinStock _strategy;

        private int _divisor;
        private long _position;
        private double _entryValue;
        private double _unrealPL;
        private double _realPL;

        #endregion
        //////////////////
        #region Properties

        public Strategy Strategy { get { return _strategy; } }
        public long Position { get { return _position; } set { _position = value; NotifyPropertyChanged("Position"); } }
        public double UnrealPL { get { return _unrealPL; } set { _unrealPL = value; NotifyPropertyChanged("UnrealPL"); NotifyPropertyChanged("NetPL"); } }
        public double RealPL { get { return _realPL; } set { _realPL = value; NotifyPropertyChanged("RealPL"); NotifyPropertyChanged("NetPL"); } }
        public double NetPL { get { return _realPL + _unrealPL; } }

        #endregion
        //////////////////
        #region Functions

        internal void AddUnit(TerrapinUnit unit)
        {
            unit.PropertyChanged += new PropertyChangedEventHandler(Unit_PropertyChanged);
        }

        void Unit_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            TerrapinUnit unit = sender as TerrapinUnit;
            if (unit != null)
            {
                if (e.PropertyName == "EntryActual")
                {
                    if (unit.Direction == Side.Long)
                    {
                        Position += (long)unit.Quantity;
                        _entryValue -= unit.Quantity * (unit.EntryActual / _divisor);
                    }
                    else
                    {
                        Position -= (long)unit.Quantity;
                        _entryValue += unit.Quantity * (unit.EntryActual / _divisor);
                    }
                    UnrealPL = _entryValue - (unit.EntryActual / _divisor * _position);
                }
                else if (e.PropertyName == "ExitActual")
                {
                    if (unit.Direction == Side.Long)
                    {
                        Position -= (long)unit.Quantity;
                        _entryValue += unit.Quantity * (unit.EntryActual / _divisor);
                    }
                    else
                    {
                        Position += (long)unit.Quantity;
                        _entryValue -= unit.Quantity * (unit.EntryActual / _divisor);
                    }
                    RealPL += unit.Profit;
                }
            }
        }

        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public void OnPriceLast(IStock stock, Objects.Finance.Prices.PriceVolume last)
        {
            UnrealPL = _entryValue + (last.Price / _divisor * _position);
        }

        #endregion
        //////////////////
        #region Enums



        #endregion

    }
}
